/*
	NUMERICKE METODY I

	Implementace Brentova algoritmu hledani korene nelinearni rovnice f(x)=0.

	Autor: Jan Schee
*/

#include <stdio.h>
#include <math.h>

#define SWAP(a,b) {double c; c=a; a=b;b=c;}

double f(double x)
{
	//return 1/x+1.5; [-1.0,-0.1]
	return exp(x)-x*x;//[-2,0]
	//return sqrt(x)-1.1;//[0,Pi]
}


double brent(double a, double b, double eps, int *cnt_bin, int *cnt_tot)
{
	double x0,x1,x2;
	double f0,f1,f2;
	double xc,xcp,fc;
	double m,fm;
	double eps1=1e-8;
	int i=0,imax=10000;

	*cnt_bin=0;
	*cnt_tot=0;
	x0=a;
	x2=b;
	x1=0.5*(x0+x2);
	xc=a;	
	
	f0=f(x0);
	f1=f(x1);
	f2=f(x2);
	
	while(i<imax && fabs(x0-x2)>eps && (fabs(f(xc))>eps) && fabs(f(x2))>eps )
	{
	
		if(fabs(f1-f2)>eps1 && fabs(f0-f1)>eps1)
		{
			xc = f1*f2*x0/((f0-f1)*(f0-f2));
			xc = xc + f0*f2*x1/((f1-f0)*(f1-f2));
			xc = xc + f0*f1*x2/((f2-f0)*(f2-f1));
		}else{
			xc = x2 - f1*(x2-x0)/(f2-f0); 
		}
		
		fc=f(xc);
	
		if(x1<xc) SWAP(x1,xc);		

		if(fc*f1<0){
			x0=xc;
			x2=x1;
		}else{
			if(f1*f2<0){
				x0=x1;
				//x2=xc;
			}else{
				x2=xc;
			}
		}
		
		*cnt_tot = (*cnt_tot) + 1;
		
		x1=(x0+x2)/2.0;

		f0=f(x0);
		f1=f(x1);
		f2=f(x2);
	
		i++;
		
		
	}

	if(fabs(f(x2))<fabs(f(xc))) return x2;

	return xc;

}


double bisekce(double a, double b, double eps, int *cnt_tot)
{
	double x0,x1;
	double f0,f1;
	double xc,fc;
	
	int i=0,imax=100;

	*cnt_tot=0;
	x0=a;
	x1=b;

	
	while(i<imax && fabs(x0-x1)>eps)
	{
		xc=(x0+x1)/2.0;
		
		f0=f(x0);
		f1=f(x1);
		
		
		fc=f(xc);

		if(fabs(fc)<eps) break;

		if(f0*fc<0) x1=xc;
		else x0=xc;

		
		*cnt_tot = (*cnt_tot) + 1;

		i++;
		
		
	}

	return xc;

}




int main()
{
	double a,b,eps;
	double r;
	int i,j;

	a=-2.0;
	b=0.0;
	eps=1e-15;

	r=brent(a,b,eps,&i,&j);

	printf("BRENT:r=%.12lf  y=%.13lf (%d,%d)\n",r,f(r),i,j);

	r=bisekce(a,b,eps,&i);

	printf("BISEC:r=%.12lf  y=%.13lf (%d)\n",r,f(r),i);

	

	return 0;
}

