#include <stdio.h>
#include <math.h>

#define FUNC(f) double (*f)(double, double) 


double func(double t, double y)
{
	return 4/(1+t*t);
}

int rk4_step(double t, double *y, double h, FUNC(f))
{
	double yk;
	double k1,k2,k3,k4;

	yk = *y;

	k1=h*f(t,yk);
	k2=h*f(t+0.5*h,yk+0.5*k1);
	k3=h*f(t+0.5*h,yk+0.5*k2);
	k4=h*f(t+h,yk+k3);

	*y = yk + 1.0/6.0*(k1+2*k2+2*k3+k4);
	
}



int main()
{
	int i;
	double y,y1,y2,t,h,ttmp,ytmp;
	double eps,err;


	t=0;
	y=0;

	h=1e-6;//initial step guess

	eps=1.0e-10;
	y1=y;
	y2=y;
	
	i=0;
	ttmp=t;
	ytmp=y;

	while(t<1.0)
	{
		printf("%lf %.8lf %.8lf\n",t,y,4*atan(t));
		while(1)
		{
		
			
			rk4_step(t,&y1,2*h,&func);
	
			rk4_step(t,&y2,h,&func);
			rk4_step(t+h,&y2,h,&func);

			err=fabs(y2-y1);

			if(err<eps){
				h=2*h;
				y2=y1;
				y=y1;
				break;
			}else{
				h=h/2;
				y1=y;
				y2=y;			
			}
		}

		if(fabs(1.0-t)<1.e-15) break;
 
		t=t+h;

		if(t>=1.0){
			while(ttmp+h>1.0)
			{			
				h=h/4;
				if(h<1e-15) break;
			}
		
			
			t=ttmp;
			y=ytmp;
			y1=y;
			y2=y;
		}else{
			ttmp=t;
			ytmp=y;						
		}
		i++;			
	}

	fprintf(stderr,"i=%d\n",i);
	return 0;
}
